@@ -114,6 +114,7 @@ def generate_residue_tuples(hierarchy,
                             include_non_linked=False,
                             backbone_only=True,
                             include_non_standard_residues=False,
+                            include_d_amino_acids=False,
                             allow_poly_ca=False,
                             # CDL specific
                             cdl_class=False,
@@ -145,6 +146,8 @@ def generate_residue_tuples(hierarchy,
     residue_lookup = ['common_amino_acid']
     if include_non_standard_residues:
       residue_lookup.append('modified_amino_acid')
+    if include_d_amino_acids:
+      residue_lookup.append('d_amino_acid')
   if backbone_only:
     backbone_asc = hierarchy.atom_selection_cache()
     backbone_sel = backbone_asc.selection(retain_selection)
@@ -222,6 +225,7 @@ def generate_protein_tuples(hierarchy,
                             include_non_linked=False,
                             backbone_only=True,
                             include_non_standard_peptides=False,
+                            include_d_amino_acids=False,
                             allow_poly_ca=False,
                             include_linked_via_restraints_manager=False,
                             # CDL specific
@@ -237,6 +241,7 @@ def generate_protein_tuples(hierarchy,
                                       include_non_linked=include_non_linked,
                                       backbone_only=backbone_only,
                                       include_non_standard_residues=include_non_standard_peptides,
+                                      include_d_amino_acids=include_d_amino_acids,
                                       allow_poly_ca=allow_poly_ca,
                                       #include_linked_via_restraints_manager=include_linked_via_restraints_manager,
                                       # CDL specific
@@ -254,6 +259,7 @@ def generate_protein_threes(hierarchy,
                             include_non_linked=False,
                             backbone_only=True,
                             include_non_standard_peptides=False,
+                            include_d_amino_acids=False,
                             include_linked_via_restraints_manager=False,
                             allow_poly_ca=False,
                             # CDL specific
@@ -268,6 +274,7 @@ def generate_protein_threes(hierarchy,
     include_non_linked=include_non_linked,
     backbone_only=backbone_only,
     include_non_standard_peptides=include_non_standard_peptides,
+    include_d_amino_acids=include_d_amino_acids,
     allow_poly_ca=allow_poly_ca,
     # include_linked_via_restraints_manager=include_linked_via_restraints_manager,
     cdl_class=cdl_class,
@@ -283,6 +290,7 @@ def generate_protein_fragments(hierarchy,
                                include_non_linked=False,
                                backbone_only=True,
                                include_non_standard_peptides=False,
+                               include_d_amino_acids=False,
                                allow_poly_ca = False,
                                # include_non_protein_linked=False, # NH2 1KYC
                                # include_linked_via_restraints_manager=False,
@@ -295,6 +303,7 @@ def generate_protein_fragments(hierarchy,
     include_non_linked=include_non_linked,
     backbone_only=backbone_only,
     include_non_standard_residues=include_non_standard_peptides,
+    include_d_amino_acids=include_d_amino_acids,
     allow_poly_ca=allow_poly_ca,
     # include_non_protein_linked=include_non_protein_linked,
     # include_linked_via_restraints_manager=include_linked_via_restraints_manager,
